import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;
import java.util.*;
import java.awt.font.*;

public class TargetCanvas extends Canvas {
    int PADDING = 5;
    int WIDTH,HEIGHT;
    float SCALEFACTOR;
    float CX, CY;
    float min,max,mid,target;
    float PIXEL;
    float HALFHEIGHT, HALFWIDTH;
    AffineTransform T, TINV;

    float value;
    float interval;
    
    Image buffer;
    Image fixedbg;
    Graphics gbuffer;

    Boolean set = false;
    Boolean confidence = false;
   
    public TargetCanvas(int width, int height, float min, float max, float target, float interval) {
	WIDTH = width;
	HEIGHT = height;
	setSize(WIDTH,HEIGHT);
	setPreferredSize(new Dimension(WIDTH,HEIGHT));
	
	this.min = min;
	this.max = max;
	this.mid = (max + min)/2.0f;
	this.target = target;
	this.interval = interval;
	if (interval> 0)
	    confidence = true;

	// Compute transformation
	SCALEFACTOR = (WIDTH - 2.0f*PADDING) / (max-min);
	
	CX = 0.5f * WIDTH;
	CY = 0.5f * HEIGHT;
	PIXEL = (1.0f / SCALEFACTOR);
	HALFHEIGHT = ((0.5f * HEIGHT - PADDING) / SCALEFACTOR);
	HALFWIDTH = (max-min)/2.0f;
	T = new AffineTransform();
	T.translate(CX,CY);
	T.scale(SCALEFACTOR,-1.0f*SCALEFACTOR);
	T.translate(-mid,0);
	try {
	    TINV = T.createInverse();
	}
	catch(Exception e) {
	    // Very bad... scale + translate was not invertible?
	    // In that case make TINV the identity.
	    TINV = new AffineTransform();
	}
    }

    public void addNotify() {
	super.addNotify();
	// Create fixed background
	fixedbg = createImage(WIDTH,HEIGHT);
	paintFixed((Graphics2D)fixedbg.getGraphics());

	// Create buffer
	buffer = createImage(WIDTH,HEIGHT);
	gbuffer = buffer.getGraphics();
    }

    public void paintFixed(Graphics2D g) {
	// Pretty rendering
	g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
	// Clear
	g.setColor(Color.black);
	g.fillRect(0,0,getWidth(),getHeight());

	// From now on, use complex coordinates
	AffineTransform saved = g.getTransform();
	g.transform(T);

	g.setColor(Color.gray);
	g.setStroke(new BasicStroke(2.0f*PIXEL));
	GeneralPath P = new GeneralPath();
	P.moveTo(this.min,0.0f);
	P.lineTo(this.max,0.0f);
	g.draw(P);

	g.setColor(Color.white);
	g.setStroke(new BasicStroke(1.0f*PIXEL));
	P = new GeneralPath();
	P.moveTo(this.min,0.0f);
	P.lineTo(this.min,(-1.0f * HALFHEIGHT));
	P.moveTo(this.max,0.0f);
	P.lineTo(this.max,(-1.0f * HALFHEIGHT));
	g.draw(P);
	g.setColor(Color.green);
	g.draw(makePointer(this.target,-1));

	// Revert to device coordinates
	g.setTransform(saved);
    }

    GeneralPath makePointer(float x, int orientation)
    {
	GeneralPath P = new GeneralPath();
	P.moveTo(x,0.0f);
	P.lineTo(x + (0.04f*HALFWIDTH),((float)orientation * 0.9f * HALFHEIGHT));
	P.lineTo(x - (0.04f*HALFWIDTH),((float)orientation * 0.9f * HALFHEIGHT));
	P.closePath();
	return P;
    }

    GeneralPath makeBracket(float x, int orientation)
    {
	GeneralPath P = new GeneralPath();
	P.moveTo(x + (float)orientation * 0.05f*HALFHEIGHT,0.35f * HALFHEIGHT);
	P.lineTo(x, 0.3f * HALFHEIGHT);
	P.lineTo(x, -0.3f * HALFHEIGHT);
	P.lineTo(x + (float)orientation * 0.05f*HALFHEIGHT,-0.35f * HALFHEIGHT);
	return P;
    }

    public void paint(Graphics g) {
        paintReal(gbuffer);
        g.drawImage(buffer, 0, 0, this);
    }
    
    public void paintReal(Graphics g2) {
	g2.drawImage(fixedbg, 0, 0, this);
	Graphics2D g = (Graphics2D)g2;
	    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
	    // From now on, use complex coordinates
	    AffineTransform saved = g.getTransform();
	    g.transform(T);	

	    if (set) {
		if (confidence) {
		    g.setStroke(new BasicStroke((float)1.0*PIXEL));
		    g.setColor(new Color(255,100,100,100));
		    g.draw(makeBracket(target-interval,1));
		    g.draw(makeBracket(target+interval,-1));
		}
		g.setStroke(new BasicStroke((float)1.0*PIXEL));
		g.setColor(Color.red);
		g.draw(makePointer(value,1));
	    }

	    // Revert to device coordinates
	    g.setTransform(saved);
    }

    public void setValue(float x)
    {
	set = true;
	value = x;
	repaint(20);
    }

    public void unSet()
    {
	set = false;
	repaint(20);
    }	

    public void setInterval(float interval)
    {
	this.interval = interval;
	repaint(20);
    }

    public void showConfidence()
    {
	confidence = true;
	repaint(20);
    }

    public void hideConfidence()
    {
	confidence = false;
	repaint(20);
    }
}
